@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget.h,v 1.18 2003/04/25 16:55:00 hadess Exp $
+ * $Id: bacon-video-widget.h,v 1.24 2003/05/21 17:42:43 dolphy Exp $
  *
  * the xine engine in a widget - header
  */
@@ -79,9 +79,12 @@ typedef struct {
 
 	void (*error) (GtkWidget *bvw, const char *message);
 	void (*eos) (GtkWidget *bvw);
+	void (*got_metadata) (GtkWidget *bvw);
 	void (*title_change) (GtkWidget *bvw, const char *title);
+	void (*channels_change) (GtkWidget *bvw);
 	void (*tick) (GtkWidget *bvw, int current_time, int stream_length,
 			int current_position);
+	void (*buffering) (GtkWidget *bvw, guint progress);
 } BaconVideoWidgetClass;
 
 GType bacon_video_widget_get_type                (void);
@@ -121,23 +124,20 @@ void bacon_video_widget_set_proprietary_plugins_path
 						 (BaconVideoWidget *bvw,
 				                  const char *path);
 
-void bacon_video_widget_set_fullscreen           (BaconVideoWidget *bvw,
-						  gboolean fullscreen);
-gboolean bacon_video_widget_is_fullscreen        (BaconVideoWidget *bvw);
-
 gboolean bacon_video_widget_can_set_volume       (BaconVideoWidget *bvw);
 void bacon_video_widget_set_volume               (BaconVideoWidget *bvw,
 						  int volume);
 int bacon_video_widget_get_volume                (BaconVideoWidget *bvw);
 
+void bacon_video_widget_set_fullscreen		 (BaconVideoWidget *bvw,
+						  gboolean fullscreen);
 void bacon_video_widget_set_show_cursor          (BaconVideoWidget *bvw,
 						  gboolean use_cursor);
 gboolean bacon_video_widget_get_show_cursor      (BaconVideoWidget *bvw);
 
 void bacon_video_widget_set_media_device	 (BaconVideoWidget *bvw,
 						  const char *path);
-gboolean bacon_video_widget_set_show_visuals	 (BaconVideoWidget *bvw,
-						  gboolean show_visuals);
+gboolean bacon_video_widget_get_auto_resize (BaconVideoWidget *bvw);
 void bacon_video_widget_set_auto_resize		 (BaconVideoWidget *bvw,
 						  gboolean auto_resize);
 
@@ -174,6 +174,22 @@ void bacon_video_widget_get_metadata		 (BaconVideoWidget *bvw,
 						  type,
 						  GValue *value);
 
+/* Visualisation functions */
+typedef enum {
+	VISUAL_SMALL,
+	VISUAL_NORMAL,
+	VISUAL_LARGE,
+	VISUAL_EXTRA_LARGE,
+} VisualsQuality;
+
+gboolean bacon_video_widget_set_show_visuals	  (BaconVideoWidget *bvw,
+						   gboolean show_visuals);
+GList *bacon_video_widget_get_visuals_list	  (BaconVideoWidget *bvw);
+gboolean bacon_video_widget_set_visuals		  (BaconVideoWidget *bvw,
+						   const char *name);
+void bacon_video_widget_set_visuals_quality	  (BaconVideoWidget *bvw,
+						   VisualsQuality quality);
+
 /* DVD functions */
 typedef enum {
 	BVW_DVD_ROOT_MENU,
