@@ -28,6 +28,7 @@
 #include <libxml/parser.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <libgnome/gnome-desktop-item.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
@@ -38,6 +39,14 @@
 #define READ_CHUNK_SIZE 8192
 #define MIME_READ_CHUNK_SIZE 1024
 
+typedef gboolean (*PlaylistCallback) (GtkPlaylist *playlist, const char *mrl,
+		gpointer data);
+
+typedef struct {
+	char *mimetype;
+	PlaylistCallback func;
+} PlaylistTypes;
+
 struct GtkPlaylistPrivate
 {
 	GladeXML *xml;
@@ -1278,6 +1287,9 @@ gtk_playlist_add_one_mrl (GtkPlaylist *playlist, const char *mrl,
 		filename_for_display = g_strdup (display_name);
 	}
 
+	D("gtk_playlist_add_one_mrl (): %s %s %s\n",
+				filename_for_display, mrl, display_name);
+
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter,
@@ -1301,7 +1313,7 @@ gtk_playlist_add_one_mrl (GtkPlaylist *playlist, const char *mrl,
 }
 
 static gboolean
-gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl)
+gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl, gpointer data)
 {
 	gboolean retval = FALSE;
 	char *contents, **lines;
@@ -1336,12 +1348,13 @@ gtk_playlist_add_m3u (GtkPlaylist *playlist, const char *mrl)
 }
 
 static gboolean
-gtk_playlist_add_asf_playlist (GtkPlaylist *playlist, const char *mrl)
+gtk_playlist_add_asf_playlist (GtkPlaylist *playlist, const char *mrl,
+		gpointer data)
 {
 	gboolean retval = FALSE;
-	char *contents, **lines;
+	char *contents, **lines, *ref;
+	int size;
 
-#if 0
 	if (my_eel_read_entire_file (mrl, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
@@ -1350,12 +1363,27 @@ gtk_playlist_add_asf_playlist (GtkPlaylist *playlist, const char *mrl)
 
 	lines = g_strsplit (contents, "\n", 0);
 	g_free (contents);
-#endif
-	return FALSE;
+
+	ref = read_ini_line_string (lines, "Ref1");
+
+	if (ref == NULL)
+		goto bail;
+
+	/* change http to mms, thanks Microsoft */
+	if (strncmp ("http", ref, 4) == 0)
+		memcpy(ref, "mmsh", 4);
+
+	retval = gtk_playlist_add_one_mrl (playlist, ref, NULL);
+	g_free (ref);
+
+bail:
+	g_strfreev (lines);
+
+	return retval;
 }
 
 static gboolean
-gtk_playlist_add_pls (GtkPlaylist *playlist, const char *mrl)
+gtk_playlist_add_pls (GtkPlaylist *playlist, const char *mrl, gpointer data)
 {
 	gboolean retval = FALSE;
 	char *contents, **lines;
@@ -1487,7 +1515,7 @@ parse_asx_entries (GtkPlaylist *playlist, char *base, xmlDocPtr doc,
 }
 
 static gboolean
-gtk_playlist_add_asx (GtkPlaylist *playlist, const char *mrl)
+gtk_playlist_add_asx (GtkPlaylist *playlist, const char *mrl, gpointer data)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
@@ -1548,7 +1576,7 @@ gtk_playlist_add_ra (GtkPlaylist *playlist, const char *mrl, gpointer data)
 	}
 
 	/* How nice, same format as m3u it seems */
-	return gtk_playlist_add_m3u (playlist, mrl);
+	return gtk_playlist_add_m3u (playlist, mrl, NULL);
 }
 
 static gboolean
@@ -1642,7 +1670,7 @@ parse_smil_entries (GtkPlaylist *playlist, char *base, xmlDocPtr doc,
 }
 
 static gboolean
-gtk_playlist_add_smil (GtkPlaylist *playlist, const char *mrl)
+gtk_playlist_add_smil (GtkPlaylist *playlist, const char *mrl, gpointer data)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
@@ -1695,78 +1723,151 @@ gtk_playlist_add_smil (GtkPlaylist *playlist, const char *mrl)
 static gboolean
 gtk_playlist_add_asf (GtkPlaylist *playlist, const char *mrl, gpointer data)
 {
-	return gtk_playlist_add_one_mrl (playlist, mrl, NULL);
-#if 0
 	if (data == NULL)
-		return gtk_playlist_add_one_mrl (playlist, mrl);
+		return gtk_playlist_add_one_mrl (playlist, mrl, NULL);
 
 	if (strncmp (data, "[Reference]", strlen ("[Reference]")) != 0)
-			return gtk_playlist_add_one_mrl (playlist, mrl);
+		return gtk_playlist_add_one_mrl (playlist, mrl, NULL);
 
-	return gtk_playlist_add_asf_playlist (playlist, mrl);
-#endif
+	return gtk_playlist_add_asf_playlist (playlist, mrl, data);
+}
 
+static gboolean
+gtk_playlist_add_desktop (GtkPlaylist *playlist, const char *mrl, gpointer data)
+{
+	GnomeDesktopItem *ditem;
+	int type;
+	gboolean retval;
+	const char *path, *display_name;
+
+	ditem = gnome_desktop_item_new_from_file (mrl, 0, NULL);
+	if (ditem == NULL)
+		return FALSE;
+
+	type = gnome_desktop_item_get_entry_type (ditem);
+	if (type != GNOME_DESKTOP_ITEM_TYPE_LINK)
+	{
+		gnome_desktop_item_unref (ditem);
+		return FALSE;
+	}
+
+	path = gnome_desktop_item_get_string (ditem, "URL");
+	if (path == NULL)
+	{
+		gnome_desktop_item_unref (ditem);
+		return FALSE;
+	}
+	display_name = gnome_desktop_item_get_localestring (ditem, "Name");
+	retval = gtk_playlist_add_mrl (playlist, path, display_name);
+	gnome_desktop_item_unref (ditem);
+
+	return retval;
+}
+
+static gboolean
+gtk_playlist_add_directory (GtkPlaylist *playlist, const char *mrl,
+		gpointer data)
+{
 	return FALSE;
 }
 
+/* These ones need a special treatment, mostly playlist formats */
+static PlaylistTypes special_types[] = {
+	{ "audio/x-mpegurl", gtk_playlist_add_m3u },
+	{ "audio/x-ms-asx", gtk_playlist_add_asx },
+	{ "audio/x-scpls", gtk_playlist_add_pls },
+	{ "application/x-smil", gtk_playlist_add_smil },
+	{ "application/x-gnome-app-info", gtk_playlist_add_desktop },
+	{ "x-directory/normal", gtk_playlist_add_directory },
+	{ "video/x-ms-wvx", gtk_playlist_add_asf_playlist },
+};
+
+/* These ones are "dual" types, might be a video, might be a playlist */
+static PlaylistTypes dual_types[] = {
+	{ "audio/x-real-audio", gtk_playlist_add_ra },
+	{ "audio/x-pn-realaudio", gtk_playlist_add_ra },
+	{ "application/vnd.rn-realmedia", gtk_playlist_add_ra },
+	{ "audio/x-pn-realaudio-plugin", gtk_playlist_add_ra },
+	{ "video/x-ms-asf", gtk_playlist_add_asf },
+	{ "video/x-ms-wmv", gtk_playlist_add_asf },
+};
+
+gboolean
+gtk_playlist_add_mrl_with_data (GtkPlaylist *playlist, const char *mrl,
+		const char *display_name)
+{
+	const char *mimetype;
+	gboolean retval;
+	gpointer data;
+	int i;
+
+	mimetype = my_gnome_vfs_get_mime_type_with_data (mrl, &data);
+	D("gtk_playlist_add_mrl_with_data adding %s, type %s", mrl, mimetype);
+	if (mimetype == NULL)
+	{
+		return gtk_playlist_add_one_mrl (playlist,
+				mrl, display_name);
+	}
+
+	for (i = 0; i < G_N_ELEMENTS(special_types); i++)
+	{
+		if (mimetype == NULL)
+			break;
+		if (strcmp (special_types[i].mimetype, mimetype) == 0)
+		{
+			retval = (* special_types[i].func) (playlist, mrl, data);
+			g_free (data);
+			return retval;
+		}
+	}
+
+	for (i = 0; i < G_N_ELEMENTS(dual_types); i++)
+	{
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0)
+		{
+			retval = (* dual_types[i].func) (playlist, mrl, data);
+			g_free (data);
+			return retval;
+		}
+	}
+
+	g_free (data);
+
+	return gtk_playlist_add_one_mrl (playlist, mrl, display_name);
+}
+
 gboolean
 gtk_playlist_add_mrl (GtkPlaylist *playlist, const char *mrl,
 		const char *display_name)
 {
 	const char *mimetype;
-	gpointer data;
 	gboolean retval = FALSE;
+	int i;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
-	mimetype = my_gnome_vfs_get_mime_type_with_data (mrl, &data);
+	mimetype = gnome_vfs_get_mime_type (mrl);
+	D("gtk_playlist_add_mrl adding %s, type %s", mrl, mimetype);
 
 	if (mimetype == NULL)
 	{
-		D("adding '%s' with no mimetype", mrl);
-		g_free (data);
-		return gtk_playlist_add_one_mrl (playlist, mrl, display_name);
-	} else if (strcmp ("audio/x-mpegurl", mimetype) == 0) {
-		g_free (data);
-		return gtk_playlist_add_m3u (playlist, mrl);
-	} else if (strcmp ("audio/x-scpls", mimetype) == 0) {
-		g_free (data);
-		return gtk_playlist_add_pls (playlist, mrl);
-	} else if (strcmp ("audio/x-ms-asx", mimetype) == 0) {
-		return gtk_playlist_add_asx (playlist, mrl);
-	} else if (strcmp ("audio/x-real-audio", mimetype) == 0
-			|| strcmp ("audio/x-pn-realaudio", mimetype) == 0
-			|| strcmp ("application/vnd.rn-realmedia",
-				mimetype) == 0
-			|| strcmp ("audio/x-pn-realaudio-plugin",
-				mimetype) == 0) {
-		retval = gtk_playlist_add_ra (playlist, mrl, data);
-		g_free (data);
-		return retval;
-	} else if (strcmp ("video/x-ms-asf", mimetype) == 0) {
-		retval = gtk_playlist_add_asf (playlist, mrl, data);
-		g_free (data);
-		return retval;
-	} else if (strcmp ("application/x-smil", mimetype) == 0) {
-		g_free (data);
-		return gtk_playlist_add_smil (playlist, mrl);
-	} else if (strcmp ("x-directory/normal", mimetype) == 0) {
-		//FIXME Load all the files in the dir ?
+		return gtk_playlist_add_mrl_with_data (playlist,
+				mrl, display_name);
 	}
 
-	g_free (data);
-
-	if (strncmp ("audio/", mimetype, 6) != 0
-			&& strncmp ("video/", mimetype, 6) != 0
-			&& strcmp ("application/x-ogg", mimetype) != 0
-			&& strcmp ("application/ogg", mimetype) != 0
-			&& strcmp ("image/png", mimetype) != 0
-			&& strcmp ("application/octet-stream", mimetype) != 0)
+	for (i = 0; i < G_N_ELEMENTS(special_types); i++)
 	{
-		//FIXME error message
-		D("not adding '%s' with mimetype '%s'",
-				mrl, mimetype);
-		return FALSE;
+		if (strcmp (special_types[i].mimetype, mimetype) == 0)
+			return (* special_types[i].func) (playlist, mrl, NULL);
+	}
+
+	for (i = 0; i < G_N_ELEMENTS(dual_types); i++)
+	{
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0)
+		{
+			return gtk_playlist_add_mrl_with_data (playlist,
+					mrl, NULL);
+		}
 	}
 
 	return gtk_playlist_add_one_mrl (playlist, mrl, display_name);
